@@ -52,8 +52,7 @@ symbol (example: `python bootstrap.py -- --hide-console`).
 Type `python bootstrap.py -- --help` to read about Spyder
 options.""")
 parser.add_argument('--gui', default=None,
-                    help="GUI toolkit: pyqt5 (for PyQt5) or pyside2 "
-                    "(for PySide2)")
+                    help="GUI toolkit: pyqt5, pyside2, pyqt6 or pyside6")
 parser.add_argument('--hide-console', action='store_true', default=False,
                     help="Hide parent console window (Windows only)")
 parser.add_argument('--safe-mode', dest="safe_mode",
@@ -71,8 +70,9 @@ parser.add_argument('spyder_options', nargs='*')
 
 args = parser.parse_args()
 
-assert args.gui in (None, 'pyqt5', 'pyside2'), \
-       "Invalid GUI toolkit option '%s'" % args.gui
+assert args.gui in (None, "pyqt5", "pyside2", "pyqt6", "pyside6"), (
+    "Invalid GUI toolkit option '%s'" % args.gui
+)
 
 
 # =============================================================================
@@ -86,7 +86,7 @@ if not args.no_install:
         prev_branch = boot_branch_file.read_text()
 
     result = subprocess.run(
-        ["git", "merge-base", "--fork-point", "master"],
+        ["git", "-C", DEVPATH, "merge-base", "--fork-point", "master"],
         capture_output=True
     )
     branch = "master" if result.stdout else "not master"
@@ -101,11 +101,14 @@ if not args.no_install:
         REPOS[DEVPATH.name]["editable"] = False
 
     for name in REPOS.keys():
-        if not REPOS[name]['editable']:
-            install_repo(name)
-            installed_dev_repo = True
-        else:
-            logger.info("%s installed in editable mode", name)
+        # Don't install the spyder-remote-services subrepo because it's not
+        # necessary on the Spyder side.
+        if name != "spyder-remote-services":
+            if not REPOS[name]['editable']:
+                install_repo(name)
+                installed_dev_repo = True
+            else:
+                logger.info("%s installed in editable mode", name)
 
 if installed_dev_repo:
     logger.info("Restarting bootstrap to pick up installed subrepos")
