@@ -12,7 +12,7 @@ Debugger Main Plugin Widget.
 import os.path as osp
 
 # Third party imports
-from qtpy.QtCore import Signal, Slot
+from qtpy.QtCore import Qt, Signal, Slot
 from qtpy.QtWidgets import QHBoxLayout, QSplitter
 
 # Local imports
@@ -39,11 +39,11 @@ class DebuggerWidgetActions:
     Inspect = 'inspect'
     EnterDebug = 'enter_debug'
     InterrupAndDebug = "interrupt_and_debug"
-    Next = "next"
-    Continue = "continue"
-    Step = "step"
-    Return = "return"
-    Stop = "stop"
+    Next = "debug current line"
+    Continue = "debug continue"
+    Step = "debug step into"
+    Return = "debug step return"
+    Stop = "debug stop"
     GotoCursor = "go to editor"
 
     # Toggles
@@ -257,10 +257,12 @@ class DebuggerWidget(ShellConnectMainWidget):
 
         next_action = self.create_action(
             DebuggerWidgetActions.Next,
-            text=_("Execute current line"),
+            text=_("Debug current line"),
             icon=self.create_icon('arrow-step-over'),
             triggered=lambda: self.debug_command("next"),
-            register_shortcut=True
+            register_shortcut=True,
+            context=Qt.ApplicationShortcut,
+            shortcut_context="_"
         )
 
         continue_action = self.create_action(
@@ -268,7 +270,9 @@ class DebuggerWidget(ShellConnectMainWidget):
             text=_("Continue execution until next breakpoint"),
             icon=self.create_icon('arrow-continue'),
             triggered=lambda: self.debug_command("continue"),
-            register_shortcut=True
+            register_shortcut=True,
+            context=Qt.ApplicationShortcut,
+            shortcut_context="_"
         )
 
         step_action = self.create_action(
@@ -276,7 +280,9 @@ class DebuggerWidget(ShellConnectMainWidget):
             text=_("Step into function or method"),
             icon=self.create_icon('arrow-step-in'),
             triggered=lambda: self.debug_command("step"),
-            register_shortcut=True
+            register_shortcut=True,
+            context=Qt.ApplicationShortcut,
+            shortcut_context="_"
         )
 
         return_action = self.create_action(
@@ -284,7 +290,9 @@ class DebuggerWidget(ShellConnectMainWidget):
             text=_("Execute until function or method returns"),
             icon=self.create_icon('arrow-step-out'),
             triggered=lambda: self.debug_command("return"),
-            register_shortcut=True
+            register_shortcut=True,
+            context=Qt.ApplicationShortcut,
+            shortcut_context="_"
         )
 
         stop_action = self.create_action(
@@ -292,7 +300,9 @@ class DebuggerWidget(ShellConnectMainWidget):
             text=_("Stop debugging"),
             icon=self.create_icon('stop_debug'),
             triggered=self.stop_debugging,
-            register_shortcut=True
+            register_shortcut=True,
+            context=Qt.ApplicationShortcut,
+            shortcut_context="_"
         )
 
         goto_cursor_action = self.create_action(
@@ -416,6 +426,7 @@ class DebuggerWidget(ShellConnectMainWidget):
                 post_mortem = False
                 executing = False
                 pdb_prompt = False
+                is_debugging = False
             else:
                 search_action.setEnabled(True)
                 search_action.setChecked(widget.finder_is_visible())
@@ -423,6 +434,7 @@ class DebuggerWidget(ShellConnectMainWidget):
                 sw = widget.shellwidget
                 executing = sw._executing
                 pdb_prompt = sw.is_waiting_pdb_input()
+                is_debugging = sw.is_debugging()
 
             enter_debug_action.setEnabled(post_mortem and not executing)
             interrupt_and_debug_action.setEnabled(executing)
@@ -438,7 +450,9 @@ class DebuggerWidget(ShellConnectMainWidget):
                 action = self.get_action(action_name)
                 action.setEnabled(pdb_prompt)
 
-            self._set_visible_control_debugger_buttons(pdb_prompt)
+            self._set_visible_control_debugger_buttons(
+                pdb_prompt or is_debugging
+            )
 
             rows = self.breakpoints_table.selectionModel().selectedRows()
             initial_row = rows[0] if rows else None
