@@ -23,20 +23,25 @@ from qtpy.QtGui import (QClipboard, QColor, QMouseEvent, QTextFormat,
 from qtpy.QtWidgets import QApplication, QMainWindow, QPlainTextEdit, QToolTip
 
 # Local imports
-from spyder.config.gui import get_font
-from spyder.config.manager import CONF
+from spyder.api.fonts import SpyderFontsMixin, SpyderFontType
+from spyder.api.widgets.mixins import SpyderWidgetMixin
 from spyder.plugins.editor.api.decoration import TextDecoration, DRAW_ORDERS
 from spyder.plugins.editor.utils.decoration import TextDecorationsManager
 from spyder.plugins.editor.widgets.completion import CompletionWidget
 from spyder.plugins.completion.api import CompletionItemKind
 from spyder.plugins.outlineexplorer.api import is_cell_header, document_cells
-from spyder.py3compat import PY3, to_text_string
+from spyder.py3compat import to_text_string
 from spyder.utils.palette import SpyderPalette
 from spyder.widgets.calltip import CallTipWidget, ToolTipWidget
 from spyder.widgets.mixins import BaseEditMixin
 
 
-class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
+class TextEditBaseWidget(
+    QPlainTextEdit,
+    BaseEditMixin,
+    SpyderFontsMixin,
+    SpyderWidgetMixin
+):
     """Text edit base widget"""
     BRACE_MATCHING_SCOPE = ('sof', 'eof')
     focus_in = Signal()
@@ -49,9 +54,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     sig_prev_cursor = Signal()
     sig_next_cursor = Signal()
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, class_parent=None):
         QPlainTextEdit.__init__(self, parent)
         BaseEditMixin.__init__(self)
+        SpyderWidgetMixin.__init__(self, class_parent=class_parent)
 
         self.has_cell_separators = False
         self.setAttribute(Qt.WA_DeleteOnClose)
@@ -88,7 +94,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.setup_completion()
 
         self.calltip_widget = CallTipWidget(self, hide_timer_on=False)
-        self.tooltip_widget = ToolTipWidget(self, as_tooltip=True)
+        self.tooltip_widget = ToolTipWidget(self)
 
         self.highlight_current_cell_enabled = False
 
@@ -121,8 +127,8 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self._current_line_block = None
 
     def setup_completion(self):
-        size = CONF.get('main', 'completion/size')
-        font = get_font()
+        size = self.get_conf('completion/size', section='main')
+        font = self.get_font(SpyderFontType.Monospace)
         self.completion_widget.setup_appearance(size, font)
 
     def set_indent_chars(self, indent_chars):
@@ -485,7 +491,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         # corruptions when saving files with certain combinations
         # of unicode chars on them (like the one attached on
         # spyder-ide/spyder#1546).
-        if os.name == 'nt' and PY3:
+        if os.name == 'nt':
             text = self.get_text('sof', 'eof')
             return text.replace('\u2028', '\n').replace('\u2029', '\n')\
                        .replace('\u0085', '\n')
@@ -509,13 +515,17 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         return []
 
     def get_selection_as_executable_code(self, cursor=None):
-        """Return selected text as a processed text,
-        to be executable in a Python/IPython interpreter"""
+        """
+        Get selected text in a way that allows other plugins to execute it.
+        """
         ls = self.get_line_separator()
 
         _indent = lambda line: len(line)-len(line.lstrip())
 
         line_from, line_to = self.get_selection_bounds(cursor)
+        line_col_from, line_col_to = self.get_selection_start_end(cursor)
+        line_from_off, line_to_off = self.get_selection_offsets(cursor)
+
         text = self.get_selected_text(cursor)
         if not text:
             return
@@ -524,7 +534,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         if len(lines) > 1:
             # Multiline selection -> eventually fixing indentation
             original_indent = _indent(self.get_text_line(line_from))
-            text = (" "*(original_indent-_indent(lines[0])))+text
+            text = (" " * (original_indent - _indent(lines[0]))) + text
 
         # If there is a common indent to all lines, find it.
         # Moving from bottom line to top line ensures that blank
@@ -533,7 +543,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         min_indent = 999
         current_indent = 0
         lines = text.split(ls)
-        for i in range(len(lines)-1, -1, -1):
+        for i in range(len(lines) - 1, -1, -1):
             line = lines[i]
             if line.strip():
                 current_indent = _indent(line)
@@ -554,15 +564,14 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             else:
                 break
 
-        # Add an EOL character after the last line of code so that it gets
-        # evaluated automatically by the console and any quote characters
-        # are separated from the triple quotes of runcell
-        lines.append(ls)
-
         # Add removed lines back to have correct traceback line numbers
         leading_lines_str = ls * lines_removed
 
-        return leading_lines_str + ls.join(lines)
+        return (
+            leading_lines_str + ls.join(lines),
+            (line_from_off, line_to_off),
+            (line_col_from, line_col_to)
+        )
 
     def get_cell_as_executable_code(self, cursor=None):
         """Return cell contents as executable code."""
@@ -577,10 +586,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         if not is_cell_header(block) and start > 0:
             block = self.document().findBlock(start - 1)
         # Get text
-        text = self.get_selection_as_executable_code(cursor)
+        text, off_pos, col_pos = self.get_selection_as_executable_code(cursor)
         if text is not None:
             text = ls * line_from + text
-        return text, block
+        return text, block, off_pos, col_pos
 
     def select_current_cell(self, cursor=None):
         """
@@ -857,28 +866,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                                 QTextCursor.KeepAnchor)
         self.setTextCursor(cursor)
 
-    def delete_line(self, cursor=None):
-        """Delete current line."""
-        if cursor is None:
-            cursor = self.textCursor()
-        if self.has_selected_text():
-            self.extend_selection_to_complete_lines()
-            start_pos, end_pos = cursor.selectionStart(), cursor.selectionEnd()
-            cursor.setPosition(start_pos)
-        else:
-            start_pos = end_pos = cursor.position()
-        cursor.beginEditBlock()
-        cursor.setPosition(start_pos)
-        cursor.movePosition(QTextCursor.StartOfBlock)
-        while cursor.position() <= end_pos:
-            cursor.movePosition(QTextCursor.EndOfBlock, QTextCursor.KeepAnchor)
-            if cursor.atEnd():
-                break
-            cursor.movePosition(QTextCursor.NextBlock, QTextCursor.KeepAnchor)
-        cursor.removeSelectedText()
-        cursor.endEditBlock()
-        self.ensureCursorVisible()
-
     def set_selection(self, start, end):
         cursor = self.textCursor()
         cursor.setPosition(start)
@@ -1184,8 +1171,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
     def position_widget_at_cursor(self, widget):
         # Retrieve current screen height
-        desktop = QApplication.desktop()
-        srect = desktop.availableGeometry(desktop.screenNumber(widget))
+        srect = self.screen().availableGeometry()
 
         left, top, right, bottom = (srect.left(), srect.top(),
                                     srect.right(), srect.bottom())
@@ -1199,10 +1185,12 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         point = self.cursorRect().bottomRight()
         point = self.calculate_real_position(point)
         point = self.mapToGlobal(point)
+
         # Move to left of cursor if not enough space on right
         widget_right = point.x() + widget.width()
         if widget_right > right:
             point.setX(point.x() - widget.width())
+
         # Push to right if not enough space on left
         if point.x() < left:
             point.setX(left)
@@ -1215,6 +1203,13 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             point = self.mapToGlobal(point)
             point.setX(x_position)
             point.setY(point.y() - widget.height())
+            delta_y = -2
+        else:
+            delta_y = 5 if sys.platform == "darwin" else 6
+
+        # Add small delta to the vertical position so that the widget is not
+        # shown too close to the text
+        point.setY(point.y() + delta_y)
 
         if ancestor is not None:
             # Useful only if we set parent to 'ancestor' in __init__
