@@ -9,26 +9,36 @@ IPython Console plugin based on QtConsole.
 """
 
 # Standard library imports
-import os
-import os.path as osp
+import sys
+from typing import List
 
 # Third party imports
 from qtpy.QtCore import Signal, Slot
 
 # Local imports
+from spyder.api.fonts import SpyderFontType
 from spyder.api.plugins import Plugins, SpyderDockablePlugin
 from spyder.api.plugin_registration.decorators import (
     on_plugin_available, on_plugin_teardown)
 from spyder.api.translations import _
+from spyder.plugins.ipythonconsole.api import (
+    IPythonConsolePyConfiguration,
+    IPythonConsoleWidgetMenus
+)
 from spyder.plugins.ipythonconsole.confpage import IPythonConsoleConfigPage
+from spyder.plugins.ipythonconsole.widgets.run_conf import IPythonConfigOptions
 from spyder.plugins.ipythonconsole.widgets.main_widget import (
-    IPythonConsoleWidget, IPythonConsoleWidgetOptionsMenus)
+    IPythonConsoleWidget
+)
 from spyder.plugins.mainmenu.api import (
     ApplicationMenus, ConsolesMenuSections, HelpMenuSections)
-from spyder.utils.programs import get_temp_dir
+from spyder.plugins.run.api import (
+    RunContext, RunExecutor, RunConfiguration,
+    ExtendedRunExecutionParameters, RunResult, run_execute)
+from spyder.plugins.editor.api.run import CellRun, FileRun, SelectionRun
 
 
-class IPythonConsole(SpyderDockablePlugin):
+class IPythonConsole(SpyderDockablePlugin, RunExecutor):
     """
     IPython Console plugin
 
@@ -38,9 +48,18 @@ class IPythonConsole(SpyderDockablePlugin):
     # This is required for the new API
     NAME = 'ipython_console'
     REQUIRES = [Plugins.Console, Plugins.Preferences]
-    OPTIONAL = [Plugins.Editor, Plugins.History, Plugins.MainMenu,
-                Plugins.Projects, Plugins.PythonpathManager,
-                Plugins.WorkingDirectory]
+    OPTIONAL = [
+        Plugins.Editor,
+        Plugins.History,
+        Plugins.MainInterpreter,
+        Plugins.MainMenu,
+        Plugins.Projects,
+        Plugins.PythonpathManager,
+        Plugins.RemoteClient,
+        Plugins.Run,
+        Plugins.StatusBar,
+        Plugins.WorkingDirectory,
+    ]
     TABIFY = [Plugins.History]
     WIDGET_CLASS = IPythonConsoleWidget
     CONF_SECTION = NAME
@@ -79,7 +98,7 @@ class IPythonConsole(SpyderDockablePlugin):
     This signal is emitted when the plugin focus changes.
     """
 
-    sig_edit_goto_requested = Signal((str, int, str), (str, int, str, bool))
+    sig_edit_goto_requested = Signal(str, int, str)
     """
     This signal will request to open a file in a given row and column
     using a code editor.
@@ -92,9 +111,6 @@ class IPythonConsole(SpyderDockablePlugin):
         Cursor starting row position.
     word: str
         Word to select on given row.
-    processevents: bool
-        True if the code editor need to process qt events when loading the
-        requested file.
     """
 
     sig_edit_new = Signal(str)
@@ -107,19 +123,6 @@ class IPythonConsole(SpyderDockablePlugin):
         Path to file.
     """
 
-    sig_pdb_state_changed = Signal(bool, dict)
-    """
-    This signal is emitted when the debugging state changes.
-
-    Parameters
-    ----------
-    waiting_pdb_input: bool
-        If the debugging session is waiting for input.
-    pdb_last_step: dict
-        Dictionary with the information of the last step done
-        in the debugging session.
-    """
-
     sig_shellwidget_created = Signal(object)
     """
     This signal is emitted when a shellwidget is connected to
@@ -152,14 +155,14 @@ class IPythonConsole(SpyderDockablePlugin):
         The shellwigdet.
     """
 
-    sig_external_spyder_kernel_connected = Signal(object)
+    sig_shellwidget_errored = Signal(object)
     """
-    This signal is emitted when we connect to an external Spyder kernel.
+    This signal is emitted when the current shellwidget failed to start.
 
     Parameters
     ----------
     shellwidget: spyder.plugins.ipyconsole.widgets.shell.ShellWidget
-        The shellwigdet that was connected to the kernel.
+        The shellwigdet.
     """
 
     sig_render_plain_text_requested = Signal(str)
@@ -206,35 +209,51 @@ class IPythonConsole(SpyderDockablePlugin):
         The new working directory path.
     """
 
+    sig_interpreter_changed = Signal(str)
+    """
+    This signal is emitted when the interpreter of the active shell widget has
+    changed.
+
+    Parameters
+    ----------
+    path: str
+        Path to the new interpreter.
+    """
+
     # ---- SpyderDockablePlugin API
     # -------------------------------------------------------------------------
     @staticmethod
     def get_name():
         return _('IPython console')
 
-    def get_description(self):
-        return _('IPython console')
+    @staticmethod
+    def get_description():
+        return _(
+            "Run Python files, cells, code and commands interactively."
+        )
 
-    def get_icon(self):
-        return self.create_icon('ipython_console')
+    @classmethod
+    def get_icon(cls):
+        return cls.create_icon('ipython_console')
 
     def on_initialize(self):
         widget = self.get_widget()
+
+        # Main widget signals
+        # Connect signal to open preferences
+        widget.sig_open_preferences_requested.connect(
+            self._open_interpreter_preferences
+        )
         widget.sig_append_to_history_requested.connect(
             self.sig_append_to_history_requested)
-        widget.sig_focus_changed.connect(self.sig_focus_changed)
         widget.sig_switch_to_plugin_requested.connect(self.switch_to_plugin)
         widget.sig_history_requested.connect(self.sig_history_requested)
         widget.sig_edit_goto_requested.connect(self.sig_edit_goto_requested)
-        widget.sig_edit_goto_requested[str, int, str, bool].connect(
-            self.sig_edit_goto_requested[str, int, str, bool])
         widget.sig_edit_new.connect(self.sig_edit_new)
-        widget.sig_pdb_state_changed.connect(self.sig_pdb_state_changed)
         widget.sig_shellwidget_created.connect(self.sig_shellwidget_created)
         widget.sig_shellwidget_deleted.connect(self.sig_shellwidget_deleted)
         widget.sig_shellwidget_changed.connect(self.sig_shellwidget_changed)
-        widget.sig_external_spyder_kernel_connected.connect(
-            self.sig_external_spyder_kernel_connected)
+        widget.sig_shellwidget_errored.connect(self.sig_shellwidget_errored)
         widget.sig_render_plain_text_requested.connect(
             self.sig_render_plain_text_requested)
         widget.sig_render_rich_text_requested.connect(
@@ -242,9 +261,123 @@ class IPythonConsole(SpyderDockablePlugin):
         widget.sig_help_requested.connect(self.sig_help_requested)
         widget.sig_current_directory_changed.connect(
             self.sig_current_directory_changed)
+        widget.sig_interpreter_changed.connect(
+            self.sig_interpreter_changed
+        )
 
-        self.sig_focus_changed.connect(self.main.plugin_focus_changed)
-        self._remove_old_std_files()
+        # Run configurations
+        self.cython_editor_run_configuration = {
+            'origin': self.NAME,
+            'extension': 'pyx',
+            'contexts': [
+                {'name': 'File'}
+            ]
+        }
+
+        self.python_editor_run_configuration = {
+            'origin': self.NAME,
+            'extension': 'py',
+            'contexts': [
+                {'name': 'File'},
+                {'name': 'Cell'},
+                {'name': 'Selection'},
+            ]
+        }
+
+        self.ipython_editor_run_configuration = {
+            'origin': self.NAME,
+            'extension': 'ipy',
+            'contexts': [
+                {'name': 'File'},
+                {'name': 'Cell'},
+                {'name': 'Selection'},
+            ]
+        }
+
+        self.executor_configuration = [
+            {
+                'input_extension': 'py',
+                'context': {'name': 'File'},
+                'output_formats': [],
+                'configuration_widget': IPythonConfigOptions,
+                'requires_cwd': True,
+                'priority': 0
+            },
+            {
+                'input_extension': 'ipy',
+                'context': {'name': 'File'},
+                'output_formats': [],
+                'configuration_widget': IPythonConfigOptions,
+                'requires_cwd': True,
+                'priority': 0
+            },
+            {
+                'input_extension': 'py',
+                'context': {'name': 'Cell'},
+                'output_formats': [],
+                'configuration_widget': None,
+                'requires_cwd': True,
+                'priority': 0
+            },
+            {
+                'input_extension': 'ipy',
+                'context': {'name': 'Cell'},
+                'output_formats': [],
+                'configuration_widget': None,
+                'requires_cwd': True,
+                'priority': 0
+            },
+            {
+                'input_extension': 'py',
+                'context': {'name': 'Selection'},
+                'output_formats': [],
+                'configuration_widget': None,
+                'requires_cwd': True,
+                'priority': 0
+            },
+            {
+                'input_extension': 'ipy',
+                'context': {'name': 'Selection'},
+                'output_formats': [],
+                'configuration_widget': None,
+                'requires_cwd': True,
+                'priority': 0
+            },
+            {
+                'input_extension': 'pyx',
+                'context': {'name': 'File'},
+                'output_formats': [],
+                'configuration_widget': IPythonConfigOptions,
+                'requires_cwd': True,
+                'priority': 0
+            },
+        ]
+
+    @on_plugin_available(plugin=Plugins.StatusBar)
+    def on_statusbar_available(self):
+        # Add status widgets
+        statusbar = self.get_plugin(Plugins.StatusBar)
+
+        pythonenv_status = self.get_widget().pythonenv_status
+        statusbar.add_status_widget(pythonenv_status)
+        pythonenv_status.register_ipythonconsole(self)
+
+        matplotlib_status = self.get_widget().matplotlib_status
+        statusbar.add_status_widget(matplotlib_status)
+        matplotlib_status.register_ipythonconsole(self)
+
+    @on_plugin_teardown(plugin=Plugins.StatusBar)
+    def on_statusbar_teardown(self):
+        # Remove status widgets
+        statusbar = self.get_plugin(Plugins.StatusBar)
+
+        pythonenv_status = self.get_widget().pythonenv_status
+        pythonenv_status.unregister_ipythonconsole(self)
+        statusbar.remove_status_widget(pythonenv_status.ID)
+
+        matplotlib_status = self.get_widget().matplotlib_status
+        matplotlib_status.unregister_ipythonconsole(self)
+        statusbar.remove_status_widget(matplotlib_status.ID)
 
     @on_plugin_available(plugin=Plugins.Preferences)
     def on_preferences_available(self):
@@ -260,12 +393,19 @@ class IPythonConsole(SpyderDockablePlugin):
         # Add signal to update actions state before showing the menu
         console_menu = mainmenu.get_application_menu(
             ApplicationMenus.Consoles)
-        console_menu.aboutToShow.connect(
-            widget.update_actions)
+        console_menu.aboutToShow.connect(widget.update_actions)
+
+        if sys.platform == "darwin":
+            # Avoid changing the aspect of the tabs context menu when it's
+            # visible and the user shows the console menu at the same time.
+            console_menu.aboutToShow.connect(
+                lambda: widget.tabwidget.menu.hide()
+            )
 
         # Main menu actions for the IPython Console
         new_consoles_actions = [
             widget.create_client_action,
+            widget.console_environment_menu,
             widget.special_console_menu,
             widget.connect_to_kernel_action
         ]
@@ -303,19 +443,14 @@ class IPythonConsole(SpyderDockablePlugin):
     def on_editor_available(self):
         editor = self.get_plugin(Plugins.Editor)
         self.sig_edit_goto_requested.connect(editor.load)
-        self.sig_edit_goto_requested[str, int, str, bool].connect(
-            self._load_file_in_editor)
         self.sig_edit_new.connect(editor.new)
-        editor.breakpoints_saved.connect(self.set_spyder_breakpoints)
-        editor.run_in_current_ipyclient.connect(self.run_script)
-        editor.run_cell_in_ipyclient.connect(self.run_cell)
-        editor.debug_cell_in_ipyclient.connect(self.debug_cell)
 
-        # Connect Editor debug action with Console
-        self.sig_pdb_state_changed.connect(editor.update_pdb_state)
-        editor.exec_in_extconsole.connect(self.run_selection)
-        editor.sig_file_debug_message_requested.connect(
-            self.print_debug_file_msg)
+        for run_config in [
+            self.python_editor_run_configuration,
+            self.ipython_editor_run_configuration,
+            self.cython_editor_run_configuration
+        ]:
+            editor.add_supported_run_configuration(run_config)
 
     @on_plugin_available(plugin=Plugins.Projects)
     def on_projects_available(self):
@@ -323,6 +458,11 @@ class IPythonConsole(SpyderDockablePlugin):
         projects.sig_project_loaded.connect(self._on_project_loaded)
         projects.sig_project_closed.connect(self._on_project_closed)
 
+    @on_plugin_available(plugin=Plugins.Run)
+    def on_run_available(self):
+        run = self.get_plugin(Plugins.Run)
+        run.register_executor_configuration(self, self.executor_configuration)
+
     @on_plugin_available(plugin=Plugins.WorkingDirectory)
     def on_working_directory_available(self):
         working_directory = self.get_plugin(Plugins.WorkingDirectory)
@@ -334,6 +474,17 @@ class IPythonConsole(SpyderDockablePlugin):
         pythonpath_manager = self.get_plugin(Plugins.PythonpathManager)
         pythonpath_manager.sig_pythonpath_changed.connect(self.update_path)
 
+    @on_plugin_available(plugin=Plugins.RemoteClient)
+    def on_remote_client_available(self):
+        remote_client = self.get_plugin(Plugins.RemoteClient)
+        remote_client.sig_server_stopped.connect(self._close_remote_clients)
+        remote_client.sig_server_renamed.connect(self._rename_remote_clients)
+
+    @on_plugin_available(plugin=Plugins.MainInterpreter)
+    def on_main_interpreter_available(self):
+        main_interpreter = self.get_plugin(Plugins.MainInterpreter)
+        main_interpreter.sig_environments_updated.connect(self._update_envs)
+
     @on_plugin_teardown(plugin=Plugins.Preferences)
     def on_preferences_teardown(self):
         # Register conf page
@@ -347,7 +498,7 @@ class IPythonConsole(SpyderDockablePlugin):
 
         # IPython documentation menu
         mainmenu.remove_item_from_application_menu(
-            IPythonConsoleWidgetOptionsMenus.Documentation,
+            IPythonConsoleWidgetMenus.Documentation,
             menu_id=ApplicationMenus.Help
          )
 
@@ -355,19 +506,14 @@ class IPythonConsole(SpyderDockablePlugin):
     def on_editor_teardown(self):
         editor = self.get_plugin(Plugins.Editor)
         self.sig_edit_goto_requested.disconnect(editor.load)
-        self.sig_edit_goto_requested[str, int, str, bool].disconnect(
-            self._load_file_in_editor)
         self.sig_edit_new.disconnect(editor.new)
-        editor.breakpoints_saved.disconnect(self.set_spyder_breakpoints)
-        editor.run_in_current_ipyclient.disconnect(self.run_script)
-        editor.run_cell_in_ipyclient.disconnect(self.run_cell)
-        editor.debug_cell_in_ipyclient.disconnect(self.debug_cell)
 
-        # Connect Editor debug action with Console
-        self.sig_pdb_state_changed.disconnect(editor.update_pdb_state)
-        editor.exec_in_extconsole.disconnect(self.run_selection)
-        editor.sig_file_debug_message_requested.disconnect(
-            self.print_debug_file_msg)
+        for run_config in [
+            self.python_editor_run_configuration,
+            self.ipython_editor_run_configuration,
+            self.cython_editor_run_configuration
+        ]:
+            editor.remove_supported_run_configuration(run_config)
 
     @on_plugin_teardown(plugin=Plugins.Projects)
     def on_projects_teardown(self):
@@ -375,6 +521,12 @@ class IPythonConsole(SpyderDockablePlugin):
         projects.sig_project_loaded.disconnect(self._on_project_loaded)
         projects.sig_project_closed.disconnect(self._on_project_closed)
 
+    @on_plugin_teardown(plugin=Plugins.Run)
+    def on_run_teardown(self):
+        run = self.get_plugin(Plugins.Run)
+        run.deregister_executor_configuration(
+            self, self.executor_configuration)
+
     @on_plugin_teardown(plugin=Plugins.WorkingDirectory)
     def on_working_directory_teardown(self):
         working_directory = self.get_plugin(Plugins.WorkingDirectory)
@@ -386,11 +538,24 @@ class IPythonConsole(SpyderDockablePlugin):
         pythonpath_manager = self.get_plugin(Plugins.PythonpathManager)
         pythonpath_manager.sig_pythonpath_changed.disconnect(self.update_path)
 
+    @on_plugin_teardown(plugin=Plugins.RemoteClient)
+    def on_remote_client_teardown(self):
+        remote_client = self.get_plugin(Plugins.RemoteClient)
+        remote_client.sig_server_stopped.disconnect(self._close_remote_clients)
+        remote_client.sig_server_renamed.disconnect(
+            self._rename_remote_clients
+        )
+
+    @on_plugin_teardown(plugin=Plugins.MainInterpreter)
+    def on_main_interpreter_teardown(self):
+        main_interpreter = self.get_plugin(Plugins.MainInterpreter)
+        main_interpreter.sig_environments_updated.disconnect(self._update_envs)
+
     def update_font(self):
         """Update font from Preferences"""
-        font = self.get_font()
-        rich_font = self.get_font(rich_text=True)
-        self.get_widget().update_font(font, rich_font)
+        font = self.get_font(SpyderFontType.Monospace)
+        app_font = self.get_font(SpyderFontType.Interface)
+        self.get_widget().update_font(font, app_font)
 
     def on_close(self, cancelable=False):
         """Perform actions when plugin is closed"""
@@ -398,38 +563,52 @@ class IPythonConsole(SpyderDockablePlugin):
         return self.get_widget().close_all_clients()
 
     def on_mainwindow_visible(self):
-        self.create_new_client(give_focus=False)
+        """
+        Connect to an existing kernel if a `kernel-*.json` file is given via
+        command line options. Otherwise create a new client.
+        """
+        cli_options = self.get_command_line_options()
+        connection_file = cli_options.connection_file
+
+        if connection_file is not None:
+            cf_path = self.get_widget().find_connection_file(connection_file)
+            if cf_path is None:
+                # Show an error if the connection file passed on the command
+                # line doesn't exist (find_connection_file returns None in that
+                # case).
+                self.create_new_client(give_focus=False)
+                client = self.get_current_client()
+                client.show_kernel_connection_error()
+            else:
+                self.create_client_for_kernel(cf_path, give_focus=False)
+        else:
+            self.create_new_client(give_focus=False)
 
     # ---- Private methods
     # -------------------------------------------------------------------------
-    def _load_file_in_editor(self, fname, lineno, word, processevents):
-        editor = self.get_plugin(Plugins.Editor)
-        editor.load(fname, lineno, word, processevents=processevents)
-
-    def _on_project_loaded(self):
-        projects = self.get_plugin(Plugins.Projects)
-        self.get_widget().update_active_project_path(
-            projects.get_active_project_path())
+    def _on_project_loaded(self, path):
+        self.get_widget().update_active_project_path(path)
 
     def _on_project_closed(self):
         self.get_widget().update_active_project_path(None)
 
-    def _remove_old_std_files(self):
-        """
-        Remove std files left by previous Spyder instances.
+    def _close_remote_clients(self, server_id):
+        self.get_widget().close_remote_clients(server_id)
 
-        This is only required on Windows because we can't
-        clean up std files while Spyder is running on that
-        platform.
-        """
-        if os.name == 'nt':
-            tmpdir = get_temp_dir()
-            for fname in os.listdir(tmpdir):
-                if osp.splitext(fname)[1] in ('.stderr', '.stdout', '.fault'):
-                    try:
-                        os.remove(osp.join(tmpdir, fname))
-                    except Exception:
-                        pass
+    def _rename_remote_clients(self, server_id):
+        self.get_widget().rename_remote_clients(server_id)
+
+    def _update_envs(self, envs):
+        self.get_widget().update_envs(envs)
+
+    def _open_interpreter_preferences(self):
+        """Open the Preferences dialog in the main interpreter section."""
+        self._main.show_preferences()
+        preferences = self._main.preferences
+        container = preferences.get_container()
+        dlg = container.dialog
+        index = dlg.get_index_by_name("main_interpreter")
+        dlg.set_current_index(index)
 
     # ---- Public API
     # -------------------------------------------------------------------------
@@ -489,6 +668,10 @@ class IPythonConsole(SpyderDockablePlugin):
         """Return the shellwidget of the current client"""
         return self.get_widget().get_current_shellwidget()
 
+    def set_current_shellwidget(self, shellwidget):
+        """Activate client associated to given shellwidget."""
+        self.get_widget().select_tab(shellwidget)
+
     def rename_client_tab(self, client, given_name):
         """
         Rename a client's tab.
@@ -506,8 +689,8 @@ class IPythonConsole(SpyderDockablePlugin):
         """
         self.get_widget().rename_client_tab(client, given_name)
 
-    def create_new_client(self, give_focus=True, filename='', is_cython=False,
-                          is_pylab=False, is_sympy=False, given_name=None):
+    def create_new_client(self, give_focus=True, filename='', special=None,
+                          given_name=None, path_to_custom_interpreter=None):
         """
         Create a new client.
 
@@ -518,18 +701,16 @@ class IPythonConsole(SpyderDockablePlugin):
             focus, False otherwise. The default is True.
         filename : str, optional
             Filename associated with the client. The default is ''.
-        is_cython : bool, optional
-            True if the client is expected to preload Cython support,
-            False otherwise. The default is False.
-        is_pylab : bool, optional
-            True if the client is expected to preload PyLab support,
-            False otherwise. The default is False.
-        is_sympy : bool, optional
-            True if the client is expected to preload Sympy support,
-            False otherwise. The default is False.
+        special : str, optional
+            Type of special support to preload. It can be "pylab", "cython",
+            "sympy", or None.
         given_name : str, optional
             Initial name displayed in the tab of the client.
             The default is None.
+        path_to_custom_interpreter : str, optional
+            Path to a custom interpreter the client should use regardless of
+            the interpreter selected in Spyder Preferences.
+            The default is None.
 
         Returns
         -------
@@ -538,10 +719,9 @@ class IPythonConsole(SpyderDockablePlugin):
         self.get_widget().create_new_client(
             give_focus=give_focus,
             filename=filename,
-            is_cython=is_cython,
-            is_pylab=is_pylab,
-            is_sympy=is_sympy,
-            given_name=given_name)
+            special=special,
+            given_name=given_name,
+            path_to_custom_interpreter=path_to_custom_interpreter)
 
     def create_client_for_file(self, filename, is_cython=False):
         """
@@ -560,8 +740,16 @@ class IPythonConsole(SpyderDockablePlugin):
         """
         self.get_widget().create_client_for_file(filename, is_cython=is_cython)
 
-    def create_client_for_kernel(self, connection_file, hostname=None,
-                                 sshkey=None, password=None):
+    def create_client_for_kernel(
+        self,
+        connection_file,
+        hostname=None,
+        sshkey=None,
+        password=None,
+        server_id=None,
+        give_focus=False,
+        can_close=True,
+    ):
         """
         Create a client connected to an existing kernel.
 
@@ -579,13 +767,30 @@ class IPythonConsole(SpyderDockablePlugin):
         password: str, optional
             Password to authenticate to the remote machine where the kernel is
             running.
+        server_id: str, optional
+            The remote server id to which this client is connected to.
+        give_focus : bool, optional
+            True if the new client should gain the window
+            focus, False otherwise. The default is True.
+        can_close: bool, optional
+            Whether the client can be closed. This is useful to prevent closing
+            the client that will be connected to a remote kernel before the
+            connection is established.
 
         Returns
         -------
-        None.
+        client: ClientWidget
+            The created client.
         """
-        self.get_widget().create_client_for_kernel(
-            connection_file, hostname, sshkey, password)
+        return self.get_widget().create_client_for_kernel(
+            connection_file,
+            hostname,
+            sshkey,
+            password,
+            server_id,
+            give_focus,
+            can_close,
+        )
 
     def get_client_for_file(self, filename):
         """Get client associated with a given file name."""
@@ -607,11 +812,80 @@ class IPythonConsole(SpyderDockablePlugin):
         self.get_widget().close_client(index=index, client=client,
                                        ask_recursive=ask_recursive)
 
+    # ---- For execution
+    @run_execute(context=RunContext.File)
+    def exec_files(
+        self,
+        input: RunConfiguration,
+        conf: ExtendedRunExecutionParameters
+    ) -> List[RunResult]:
+
+        exec_params = conf['params']
+        cwd_opts = exec_params['working_dir']
+        params: IPythonConsolePyConfiguration = exec_params['executor_params']
+
+        run_input: FileRun = input['run_input']
+        filename = run_input['path']
+        wdir = cwd_opts['path']
+        args = params['python_args']
+        post_mortem = params['post_mortem']
+        current_client = params['current']
+        clear_variables = params['clear_namespace']
+        console_namespace = params['console_namespace']
+        run_method = params.get('run_method', 'runfile')
+
+        self.run_script(
+            filename,
+            wdir,
+            args,
+            post_mortem,
+            current_client,
+            clear_variables,
+            console_namespace,
+            method=run_method,
+        )
+
+        return []
+
+    @run_execute(context=RunContext.Selection)
+    def exec_selection(
+        self,
+        input: RunConfiguration,
+        conf: ExtendedRunExecutionParameters
+    ) -> List[RunResult]:
+
+        run_input: SelectionRun = input['run_input']
+        text = run_input['selection']
+        self.run_selection(text)
+
+    @run_execute(context=RunContext.Cell)
+    def exec_cell(
+        self,
+        input: RunConfiguration,
+        conf: ExtendedRunExecutionParameters
+    ) -> List[RunResult]:
+
+        run_input: CellRun = input['run_input']
+        cell_text = run_input['cell']
+
+        if run_input['copy']:
+            self.run_selection(cell_text)
+            return
+
+        cell_name = run_input['cell_name']
+        filename = run_input['path']
+
+        exec_params = conf['params']
+        params: IPythonConsolePyConfiguration = exec_params['executor_params']
+        run_method = params.get('run_method', 'runcell')
+        self.run_cell(cell_text, cell_name, filename,
+                      method=run_method)
+
     # ---- For execution and debugging
-    def run_script(self, filename, wdir, args='', debug=False,
+    def run_script(self, filename, wdir, args='',
                    post_mortem=False, current_client=True,
                    clear_variables=False, console_namespace=False,
-                   focus_to_editor=True):
+                   method=None):
         """
         Run script in current or dedicated client.
 
@@ -623,9 +897,6 @@ class IPythonConsole(SpyderDockablePlugin):
             Working directory from where the file should be run.
         args : str, optional
             Arguments defined to run the file.
-        debug : bool, optional
-            True if the run if for debugging the file,
-            False for just running it.
         post_mortem : bool, optional
             True if in case of error the execution should enter in
             post-mortem mode, False otherwise.
@@ -637,8 +908,9 @@ class IPythonConsole(SpyderDockablePlugin):
             False otherwise.
         console_namespace : bool, optional
             True if the console namespace should be used, False otherwise.
-        focus_to_editor: bool, optional
-            Leave focus in the editor after execution.
+        method : str or None
+            Method to run the file. It must accept the same arguments as
+            `runfile`.
 
         Returns
         -------
@@ -649,36 +921,27 @@ class IPythonConsole(SpyderDockablePlugin):
             filename,
             wdir,
             args,
-            debug,
             post_mortem,
             current_client,
             clear_variables,
             console_namespace,
-            focus_to_editor)
+            method
+        )
 
-    def run_cell(self, code, cell_name, filename, run_cell_copy,
-                 focus_to_editor, function='runcell'):
+    def run_cell(self, code, cell_name, filename, method='runcell'):
         """
         Run cell in current or dedicated client.
 
         Parameters
         ----------
         code : str
-            Piece of code to run that corresponds to a cell in case
-            `run_cell_copy` is True.
+            Piece of code to run that corresponds to a cell.
         cell_name : str or int
             Cell name or index.
         filename : str
             Path of the file where the cell to execute is located.
-        run_cell_copy : bool
-            True if the cell should be executed line by line,
-            False if the provided `function` should be used.
-        focus_to_editor: bool
-            Whether to give focus to the editor after running the cell. If
-            False, focus is given to the console.
-        function : str, optional
-            Name handler of the kernel function to be used to execute the cell
-            in case `run_cell_copy` is False.
+        method : str, optional
+            Name handler of the kernel function to be used to execute the cell.
             The default is 'runcell'.
 
         Returns
@@ -686,38 +949,7 @@ class IPythonConsole(SpyderDockablePlugin):
         None.
         """
         self.sig_unmaximize_plugin_requested.emit()
-        self.get_widget().run_cell(
-            code, cell_name, filename, run_cell_copy, focus_to_editor,
-            function=function)
-
-    def debug_cell(self, code, cell_name, filename, run_cell_copy,
-                   focus_to_editor):
-        """
-        Debug current cell.
-
-        Parameters
-        ----------
-        code : str
-            Piece of code to run that corresponds to a cell in case
-            `run_cell_copy` is True.
-        cell_name : str or int
-            Cell name or index.
-        filename : str
-            Path of the file where the cell to execute is located.
-        run_cell_copy : bool
-            True if the cell should be executed line by line,
-            False if the `debugcell` kernel function should be used.
-        focus_to_editor: bool
-            Whether to give focus to the editor after debugging the cell. If
-            False, focus is given to the console.
-
-        Returns
-        -------
-        None.
-        """
-        self.sig_unmaximize_plugin_requested.emit()
-        self.get_widget().debug_cell(code, cell_name, filename, run_cell_copy,
-                                     focus_to_editor)
+        self.get_widget().run_cell(code, cell_name, filename, method=method)
 
     def execute_code(self, lines, current_client=True, clear_variables=False):
         """
@@ -743,51 +975,10 @@ class IPythonConsole(SpyderDockablePlugin):
             current_client=current_client,
             clear_variables=clear_variables)
 
-    def run_selection(self, lines, focus_to_editor=True):
+    def run_selection(self, lines):
         """Execute selected lines in the current console."""
         self.sig_unmaximize_plugin_requested.emit()
-        self.get_widget().execute_code(lines, set_focus=not focus_to_editor)
-
-    def stop_debugging(self):
-        """Stop debugging in the current console."""
-        self.sig_unmaximize_plugin_requested.emit()
-        self.get_widget().stop_debugging()
-
-    def get_pdb_state(self):
-        """Get debugging state of the current console."""
-        return self.get_widget().get_pdb_state()
-
-    def get_pdb_last_step(self):
-        """Get last pdb step of the current console."""
-        return self.get_widget().get_pdb_last_step()
-
-    def pdb_execute_command(self, command, focus_to_editor):
-        """
-        Send command to the pdb kernel if possible.
-
-        Parameters
-        ----------
-        command : str
-            Command to execute by the pdb kernel.
-        focus_to_editor: bool
-            Leave focus in editor after the command is executed.
-
-        Returns
-        -------
-        None.
-        """
-        self.sig_unmaximize_plugin_requested.emit()
-        self.get_widget().pdb_execute_command(command, focus_to_editor)
-
-    def print_debug_file_msg(self):
-        """
-        Print message in the current console when a file can't be closed.
-
-        Returns
-        -------
-        None.
-        """
-        self.get_widget().print_debug_file_msg()
+        self.get_widget().execute_code(lines)
 
     # ---- For working directory and path management
     def set_current_client_working_directory(self, directory):
@@ -833,10 +1024,6 @@ class IPythonConsole(SpyderDockablePlugin):
         """
         self.get_widget().save_working_directory(dirname)
 
-    def update_working_directory(self):
-        """Update working directory to console current working directory."""
-        self.get_widget().update_working_directory()
-
     def update_path(self, path_dict, new_path_dict):
         """
         Update path on consoles.
@@ -857,10 +1044,6 @@ class IPythonConsole(SpyderDockablePlugin):
         """
         self.get_widget().update_path(path_dict, new_path_dict)
 
-    def set_spyder_breakpoints(self):
-        """Set Spyder breakpoints into all clients"""
-        self.get_widget().set_spyder_breakpoints()
-
     def restart(self):
         """
         Restart the console.
