@@ -1039,8 +1039,10 @@ def test_set_elapsed_time(ipyconsole, qtbot):
 
 
 @flaky(max_runs=3)
-@pytest.mark.skipif(sys.platform == 'darwin',
-                    reason="Fails sometimes on macOS")
+@pytest.mark.skipif(
+    not sys.platform.startswith("linux"),
+    reason="Fails frequently on Windows and macOS"
+)
 def test_kernel_crash(ipyconsole, qtbot):
     """Test that we show an error message when a kernel crash occurs."""
     # Create an IPython kernel config file with a bad config
@@ -1363,7 +1365,10 @@ def test_conda_env_activation(ipyconsole, qtbot):
 
 @flaky(max_runs=3)
 @pytest.mark.parametrize("external_interpreter", [True, False])
-@pytest.mark.skipif(os.name == 'nt', reason="no SIGTERM on Windows")
+@pytest.mark.skipif(
+    not sys.platform.startswith("linux"),
+    reason="Fails frequently on macOS and no SIGTERM on Windows"
+)
 def test_kernel_kill(ipyconsole, qtbot, external_interpreter):
     """
     Test that the kernel correctly restarts after a kill.
@@ -1656,6 +1661,9 @@ def test_recursive_pdb(ipyconsole, qtbot):
     assert control.toPlainText().split()[-2:] == ["In", "[3]:"]
 
 
+@pytest.mark.skipif(
+    sys.version_info[:2] == (3, 8), reason="Fails in Python 3.8"
+)
 def test_pdb_magics_are_recursive(ipyconsole, qtbot, tmp_path):
     """
     Check that calls to Pdb magics start a recursive debugger when called in
@@ -2411,5 +2419,60 @@ def test_restore_tmpdir(ipyconsole, qtbot, tmp_path):
     assert shell.get_value('tmpdir') == system_tmpdir
 
 
+def test_filter_frames_in_tracebacks(ipyconsole, qtbot, tmp_path):
+    """Check that we filter some unnecessary frames in tracebacks."""
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    control = shell._control
+    qtbot.waitUntil(
+        lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT
+    )
+
+    # Write code to file
+    file = tmp_path / 'test_filter_frames.py'
+    file.write_text("# Comment\n1/0")
+
+    # Filename in the format used when running magics from the main toolbar
+    fname = str(file).replace('\\', '/')
+
+    # Run file
+    with qtbot.waitSignal(shell.executed):
+        shell.execute(f"%runfile {fname}")
+
+    # Check frames related to spyder-kernels are not displayed
+    assert "spyder_kernels" not in control.toPlainText()
+
+    # Switch to %xmode plain and check again
+    with qtbot.waitSignal(shell.executed):
+        shell.execute("%xmode plain")
+
+    with qtbot.waitSignal(shell.executed):
+        shell.execute(f"%runfile {fname}")
+
+    assert "spyder_kernels" not in control.toPlainText()
+
+    # Switch back to %xmode verbose (the default)
+    with qtbot.waitSignal(shell.executed):
+        shell.execute("%xmode verbose")
+
+    # Check we don't display a traceback when exiting our debugger if it's
+    # started with breakpoint().
+    file.write_text("# Comment\nbreakpoint()")
+    with qtbot.waitSignal(shell.executed):
+        shell.execute(f"%runfile {fname}")
+
+    shell.clear_console()
+    empty_console_text = (
+        "\n\nIPdb [2]: " if os.name == "nt" else "\nIPdb [2]: "
+    )
+    qtbot.waitUntil(lambda: empty_console_text == control.toPlainText())
+
+    with qtbot.waitSignal(shell.executed):
+        qtbot.keyClicks(control, '!exit')
+        qtbot.keyClick(control, Qt.Key_Enter)
+
+    assert "BdbQuit" not in control.toPlainText()
+
+
 if __name__ == "__main__":
     pytest.main()
