@@ -23,8 +23,8 @@ from jupyter_client.connect import find_connection_file
 from jupyter_core.paths import jupyter_config_dir
 import qstylizer.style
 from qtconsole.svg import save_svg, svg_to_clipboard
-from qtpy.QtCore import Signal, Slot
-from qtpy.QtGui import QColor, QKeySequence
+from qtpy.QtCore import Qt, Signal, Slot
+from qtpy.QtGui import QColor, QFontMetrics, QKeySequence
 from qtpy.QtPrintSupport import QPrintDialog, QPrinter
 from qtpy.QtWidgets import (
     QApplication, QHBoxLayout, QLabel, QMessageBox, QVBoxLayout, QWidget)
@@ -108,6 +108,11 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
     """
 
     # Signals
+    sig_open_preferences_requested = Signal()
+    """
+    Signal to open the main interpreter preferences.
+    """
+
     sig_append_to_history_requested = Signal(str, str)
     """
     This signal is emitted when the plugin requires to add commands to a
@@ -322,18 +327,10 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
 
         # Info widget
         if self.enable_infowidget:
-            from spyder.widgets.browser import FrameWebView
-
-            self.infowidget = FrameWebView(self)
-            if WEBENGINE:
-                self.infowidget.page().setBackgroundColor(
-                    QColor(MAIN_BG_COLOR))
-            else:
-                self.infowidget.setStyleSheet(
-                    "background:{}".format(MAIN_BG_COLOR))
-            layout.addWidget(self.infowidget)
+            self._infowidget = self._create_info_widget()
+            layout.addWidget(self._infowidget)
         else:
-            self.infowidget = None
+            self._infowidget = None
 
         # Label to inform users how to get out of the pager
         self.pager_label = QLabel(_("Press <b>Q</b> to exit pager"), self)
@@ -388,6 +385,8 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
         self.pythonenv_status.sig_interpreter_changed.connect(
             self.sig_interpreter_changed
         )
+        self.pythonenv_status.sig_open_preferences_requested.connect(
+            self.sig_open_preferences_requested)
 
         # Initial value for the current working directory
         self._current_working_directory = get_home_dir()
@@ -1086,6 +1085,19 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
 
     # ---- Private methods
     # -------------------------------------------------------------------------
+    def _create_info_widget(self):
+        from spyder.widgets.browser import FrameWebView
+
+        infowidget = FrameWebView(self)
+        if WEBENGINE:
+            infowidget.page().setBackgroundColor(
+                QColor(MAIN_BG_COLOR))
+        else:
+            infowidget.setStyleSheet(
+                "background:{}".format(MAIN_BG_COLOR)
+            )
+        return infowidget
+
     def _change_client_conf(self, client, client_conf_func, value):
         """
         Change a client configuration option, taking into account if it is
@@ -1195,9 +1207,13 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
         for env_key, env_info in self.envs.items():
             env_name = env_key.split()[-1]
             path_to_interpreter, python_version = env_info
+            fm = QFontMetrics(self.font())
+            env_elided = fm.elidedText(
+                env_key, Qt.ElideMiddle, 250
+            )
 
             # Text for actions
-            text = f"{env_key} ({python_version})"
+            text = f"{env_elided} ({python_version})"
 
             # Change text in case env is the default or internal (i.e. same as
             # Spyder) one
@@ -1228,6 +1244,7 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
                 ),
                 overwrite=True,
                 register_action=False,
+                tip=text,
             )
 
             # Add default env as the first entry in the menu
@@ -1346,22 +1363,52 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
         # Render consoles menu and submenus
         self.console_environment_menu.render()
 
+    # ---- Public API
+    # -------------------------------------------------------------------------
     def find_connection_file(self, connection_file):
         """Fix connection file path."""
         cf_path = osp.dirname(connection_file)
         cf_filename = osp.basename(connection_file)
+
         # To change a possible empty string to None
         cf_path = cf_path if cf_path else None
-        connection_file = find_connection_file(filename=cf_filename,
-                                               path=cf_path)
-        if os.path.splitext(connection_file)[1] != ".json":
+
+        # This error is raised when find_connection_file can't find the file
+        try:
+            connection_file = find_connection_file(
+                filename=cf_filename, path=cf_path
+            )
+        except OSError:
+            connection_file = None
+
+        if connection_file and os.path.splitext(connection_file)[1] != ".json":
             # There might be a file with the same id in the path.
             connection_file = find_connection_file(
-                filename=cf_filename + ".json", path=cf_path)
+                filename=cf_filename + ".json", path=cf_path
+            )
+
         return connection_file
 
-    # ---- Public API
-    # -------------------------------------------------------------------------
+    @property
+    def infowidget(self):
+        """
+        This is necessary to prevent an error when, in some situations, Python
+        garbage collects _infowidget.
+
+        Notes
+        -----
+        * See spyder-ide/spyder#21509 and spyder-ide/spyder#23529
+        """
+        try:
+            # We need to call a method to detect if the object was garbage
+            # collected and trigger the RuntimeError we want to catch here.
+            self._infowidget.isVisible()
+            return self._infowidget
+        except RuntimeError:
+            self._infowidget = self._create_info_widget()
+            return self._infowidget
+        except AttributeError:
+            return None
 
     # ---- General
     # -------------------------------------------------------------------------
@@ -1369,7 +1416,7 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
         self._font = font
         self._app_font = app_font
 
-        if self.enable_infowidget:
+        if self.enable_infowidget and self.infowidget is not None:
             self.infowidget.set_font(app_font)
 
         for client in self.clients:
@@ -1403,18 +1450,11 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
                 client.set_info_page()
                 client.shellwidget.hide()
                 client.layout.addWidget(self.infowidget)
-                self.infowidget.show()
+                if self.infowidget is not None:
+                    self.infowidget.show()
             else:
-                if self.enable_infowidget:
-                    try:
-                        self.infowidget.hide()
-                    except RuntimeError:
-                        # Needed to handle the possible scenario where the
-                        # `infowidget` (`FrameWebView`) related C/C++ object
-                        # has been already deleted when trying to hide it.
-                        # See spyder-ider/spyder#21509
-                        self.enable_infowidget = False
-                        self.infowidget = None
+                if self.enable_infowidget and self.infowidget is not None:
+                    self.infowidget.hide()
                 client.shellwidget.show()
 
             # Get reference for the control widget of the selected tab
@@ -1726,7 +1766,8 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
         return client
 
     def create_client_for_kernel(self, connection_file, hostname, sshkey,
-                                 password, server_id=None, can_close=True):
+                                 password, server_id=None, give_focus=False,
+                                 can_close=True):
         """Create a client connected to an existing kernel."""
         given_name = None
         master_client = None
@@ -1772,6 +1813,7 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
             additional_options=self.additional_options(),
             handlers=self.registered_spyder_kernel_handlers,
             server_id=server_id,
+            give_focus=give_focus,
             can_close=can_close,
         )
 
@@ -1849,9 +1891,10 @@ class IPythonConsoleWidget(PluginMainWidget, CachedKernelMixin):
         self.master_clients -= 1
 
         # Rename client tab with filename
-        client.allow_rename = False
-        tab_text = self.disambiguate_fname(filename)
-        self.rename_client_tab(client, tab_text)
+        if client is not None:
+            client.allow_rename = False
+            tab_text = self.disambiguate_fname(filename)
+            self.rename_client_tab(client, tab_text)
 
         return client
 
