@@ -203,6 +203,8 @@ class FigureBrowser(QWidget, SpyderWidgetMixin):
         splitter.setStyleSheet(
             f"border-radius: {SpyderPalette.SIZE_BORDER_RADIUS}"
         )
+        self.splitter.setChildrenCollapsible(False)
+        self.splitter.splitterMoved.connect(self._on_splitter_moved)
 
         self.stack_layout = QStackedLayout()
         self.stack_layout.addWidget(splitter)
@@ -212,6 +214,12 @@ class FigureBrowser(QWidget, SpyderWidgetMixin):
         self.stack_layout.setSpacing(0)
         self.setContentsMargins(0, 0, 0, 0)
 
+    def _on_splitter_moved(self):
+        total_width = self.splitter.width()
+        min_width_percentage = 0.55
+        min_width = int(total_width * min_width_percentage)
+        self.figviewer.setMinimumWidth(min_width)
+
     def _update_zoom_value(self, value):
         """
         Used in testing.
@@ -847,6 +855,12 @@ class ThumbnailScrollBar(QFrame):
                     self.scene.insertWidget(i - 1, dropped_thumbnail)
                     break
 
+        # Recreate thumbnails list to take into account the new order
+        # Fixes spyder-ide/spyder#22458
+        self._thumbnails = []
+        for i in range(n_thumbnails):
+            self._thumbnails.append(self.scene.itemAt(i).widget())
+
         event.accept()
 
     # ---- Save Figure
@@ -1041,7 +1055,7 @@ class ThumbnailScrollBar(QFrame):
         if thumbnail in self._thumbnails:
             self._thumbnails.remove(thumbnail)
 
-        # Select a new thumbnail if any :
+        # Select a new thumbnail, if any
         if thumbnail == self.current_thumbnail:
             if len(self._thumbnails) > 0:
                 self.set_current_index(
