@@ -16,15 +16,26 @@ from spyder.utils.environ import get_user_env
 
 
 def check_path(path):
-    """Check that `path` is not a [site|dist]-packages folder."""
+    """
+    Check that `path` is not a [site|dist]-packages folder or a Conda
+    distribution `pkgs` directory.
+    """
+    pattern_string = r'(ana|mini|micro)(conda|mamba|forge)\d*(/base)*/pkgs'
     if os.name == 'nt':
-        pattern = re.compile(r'.*(l|L)ib/(site|dist)-packages.*')
-    else:
-        pattern = re.compile(
-            r'.*(lib|lib64)/'
-            '(python|python\d+|python\d+\.\d+)/'
-            '(site|dist)-packages.*'
+        path_appdata = os.getenv("APPDATA").replace('\\', '/')
+        pattern_string = (
+            f'.*({pattern_string}'
+            r'|(l|L)ib/(site|dist)-packages.*'
+            f'|(AppData|{path_appdata})/Roaming/Python).*'
         )
+    else:
+        pattern_string = (
+            f'.*({pattern_string}'
+            r'|(lib|lib64)/'
+            r'(python|python\d+|python\d+\.\d+)/'
+            r'(site|dist)-packages).*'
+        )
+    pattern = re.compile(pattern_string, re.IGNORECASE)
 
     path_norm = path.replace('\\', '/')
     return pattern.match(path_norm) is None
