@@ -12,7 +12,7 @@ from spyder.plugins.remoteclient import SPYDER_REMOTE_VERSION
 SERVER_ENV = "spyder-remote"
 PACKAGE_NAME = "spyder-remote-services"
 SCRIPT_URL = (
-    f"https://raw.githubusercontent.com/spyder-ide/{PACKAGE_NAME}/master/scripts"
+    f"https://raw.githubusercontent.com/spyder-ide/{PACKAGE_NAME}/main/scripts"
 )
 
 
@@ -24,7 +24,7 @@ def get_installer_command(platform: str) -> str:
         return '\n'  # server should be aready installed in the test environment
 
     return (
-        f'"${{SHELL}}" <(curl -L {SCRIPT_URL}/installer.sh) '
+        f'"`which bash`" <(curl -L {SCRIPT_URL}/installer.sh) '
         f'"{SPYDER_REMOTE_VERSION}" "{SPYDER_KERNELS_VERSION}"'
     )
 
